# Load a Ptolemy II model from file "sis100u_alternativ.moml"
# and store a handle to the model into variable hModel :
set hModel [ ptloadmodel "sis100u_alternativ.moml" ]

# Read the parameter "controlTimeConstant_s" of the actor "AmplitudeControl",
# which is inside the actor "aCavity", which in turn is inside the actor "Cavities".
# "Cavities" is a top-level actor.
# Store the value of the parameter into variable timeACNominal :
set timeACNominal  [ ptgetparameter $hModel "Cavities.aCavity.AmplitudeControl.controlTimeConstant_s"        ]

set gainACNominal  [ ptgetparameter $hModel "Cavities.aCavity.AmplitudeControl.controlGain"                  ]
set gainAPCNominal [ ptgetparameter $hModel "Cavities.aCavity.AmplitudeControl.pilotGain"                    ]
set timeBCNominal  [ ptgetparameter $hModel "Cavities.aCavity.BiasControl.biasCurrentTimeConstant_s"         ]
set gainBCPNominal [ ptgetparameter $hModel "Cavities.aCavity.BiasControl.proportionalGain_HzPerDegree"      ]
set gainBCINominal [ ptgetparameter $hModel "Cavities.aCavity.BiasControl.integralGain_HzPerDegreePerSecond" ]
puts "\n*** Generating 14 cavities\n";

# Create 14 instances (that is, 13 copies) of the actor "aCavity" inside the actor "Cavities".
# The 14 instances will be named "aCavity_0", "aCavity_1", ..., "aCavity_13".
# Store a list of the names of all 14 instances into the variable lsGeneratedNames :
set lsGeneratedNames [ ptgenerate $hModel "Cavities.aCavity" 14 ]

puts "\n*** Cavity parameters deviate by up to +-20%\n"

# Set the parameter "minimumVoltage_V" of the actor "ControlAttachment"
# inside the instance "Cavities.aCavity_0" to 0.0 :
ptsetparameter $hModel "Cavities.aCavity_0.ControlAttachment.minimumVoltage_V" 0.0

ptsetparameter $hModel "Cavities.aCavity_0.ControlAttachment.voltageSetpointOffset_V" 0.0
ptsetparameter $hModel "Cavities.aCavity_1.ControlAttachment.voltageSetpointOffset_V" 20000.0
ptsetparameter $hModel "Cavities.aCavity_2.ControlAttachment.voltageSetpointOffset_V" 40000.0
ptsetparameter $hModel "Cavities.aCavity_3.ControlAttachment.voltageSetpointOffset_V" 60000.0
ptsetparameter $hModel "Cavities.aCavity_4.ControlAttachment.voltageSetpointOffset_V" 80000.0
ptsetparameter $hModel "Cavities.aCavity_5.ControlAttachment.voltageSetpointOffset_V" 100000.0
ptsetparameter $hModel "Cavities.aCavity_6.ControlAttachment.voltageSetpointOffset_V" 120000.0
ptsetparameter $hModel "Cavities.aCavity_7.ControlAttachment.voltageSetpointOffset_V" 140000.0
ptsetparameter $hModel "Cavities.aCavity_8.ControlAttachment.voltageSetpointOffset_V" 160000.0
ptsetparameter $hModel "Cavities.aCavity_9.ControlAttachment.voltageSetpointOffset_V" 180000.0
ptsetparameter $hModel "Cavities.aCavity_10.ControlAttachment.voltageSetpointOffset_V" 200000.0
ptsetparameter $hModel "Cavities.aCavity_11.ControlAttachment.voltageSetpointOffset_V" 220000.0
ptsetparameter $hModel "Cavities.aCavity_12.ControlAttachment.voltageSetpointOffset_V" 240000.0
ptsetparameter $hModel "Cavities.aCavity_13.ControlAttachment.voltageSetpointOffset_V" 260000.0
proc fnRandUniform { { rLowerBound 0.0 } { rUpperBound 1.0 } } { return [ expr { $rLowerBound + ( $rUpperBound - $rLowerBound ) * rand() } ] }
foreach sName $lsGeneratedNames {
	set rYErr [ fnRandUniform -0.2 0.2 ]
	set rRErr [ fnRandUniform -0.2 0.2 ]
	set rLErr [ fnRandUniform -0.2 0.2 ]
	set rCErr [ fnRandUniform -0.2 0.2 ]
	set rTFactor [ fnRandUniform  1.0 1.2 ]; set timeAC         [ expr { $rTFactor * $timeACNominal  } ]
	set rGFactor [ fnRandUniform  0.8 1.0 ]; set gainAC         [ expr { $rGFactor * $gainACNominal  } ]
	set rGFactor [ fnRandUniform  0.8 1.0 ]; set gainAPC        [ expr { $rGFactor * $gainAPCNominal } ]
	set rTFactor [ fnRandUniform  1.0 1.2 ]; set timeBC         [ expr { $rTFactor * $timeBCNominal  } ]
	set rGFactor [ fnRandUniform  0.8 1.0 ]; set gainBCPNominal [ expr { $rGFactor * $gainBCPNominal } ]
	set rGFactor [ fnRandUniform  0.8 1.0 ]; set gainBCINominal [ expr { $rGFactor * $gainBCINominal } ]
	ptsetparameter $hModel "Cavities.${sName}.Generator.relativeTransconductanceError"       $rYErr
	ptsetparameter $hModel "Cavities.${sName}.CavityPlantModel.relativeResistanceError"      $rRErr
	ptsetparameter $hModel "Cavities.${sName}.CavityPlantModel.relativeInductanceError"      $rLErr
	ptsetparameter $hModel "Cavities.${sName}.CavityPlantModel.relativeCapacitanceError"     $rCErr
	ptsetparameter $hModel "Cavities.${sName}.AmplitudeControl.controlTimeConstant_s"        $timeAC
	ptsetparameter $hModel "Cavities.${sName}.AmplitudeControl.controlGain"                  $gainAC
	ptsetparameter $hModel "Cavities.${sName}.AmplitudeControl.pilotGain"                    $gainAPC
	ptsetparameter $hModel "Cavities.${sName}.BiasControl.biasCurrentTimeConstant_s"         $timeBC
	ptsetparameter $hModel "Cavities.${sName}.BiasControl.proportionalGain_HzPerDegree"      $gainBCPNominal
	ptsetparameter $hModel "Cavities.${sName}.BiasControl.integralGain_HzPerDegreePerSecond" $gainBCINominal
}
ptsetparameter $hModel "initialFrequency_Hz"              1570590
ptsetparameter $hModel "initialVoltage_V"                 30520
ptsetparameter $hModel "initialFrequencyEstimate_Hz"      510
ptsetparameter $hModel "CentralControlSystem.inputFile" "ramp_238u28+_280kV.csv"
ptsetparameter $hModel "CSVRecordWriter.fileOrURL"      "1670-00010.csv"
ptsetparameter $hModel "BeamPhaseDDS.phaseNoise_degrees" 1.0
ptsetparameter $hModel "BPM.measurementNoise_degrees" 3.0
foreach sName $lsGeneratedNames { ptsetparameter $hModel "Cavities.${sName}.CavityDDS.phaseNoise_degrees" 1.0 }
ptsetparameter $hModel "BPM.Gaussian.seed" 1680
puts "\n*** Starting model execution...\n";

# Start the model execution in Ptolemy II.
# This call will block until the execution finishes.
ptrunmodel $hModel;

puts "\n\n\n"
