/*
 Copyright (c) 2011 Technische Universitt Darmstadt.
 
 Permission is hereby granted to use, copy, modify, and distribute this
 software, provided that this copyright notice remains intact.
 */

package gsi.rf.data.csv.parser;

import gsi.rf.data.ParseException;

/**
 * This class represents an exception that is thrown
 * due to an invalid or unrecognized character in a data file.
 * 
 * This class used to be called InvalidCharacterException,
 * but was renamed to conform with naming standards in other Java projects.
 * 
 * This exception class should be used for exceptions due to malformed input ONLY,
 * not for exceptions resulting from programming mistakes.
 * 
 * @author Christopher Spies, TU Darmstadt
 * @since 2011-05-24
 * @version 2.0.0.0
 */
public class IllegalCharacterException extends ParseException {
	
	////////////////////
	// private constants
	
	/**
	 * The Serial Version UID of this class, required by all serializable classes.
	 */
	private static final long serialVersionUID = 917675977503404098L;

	/**
	 * The character to be shown in place of non-printable control characters.
	 * 
	 * This character will be shown in the exception message in place of non-printable control characters.
	 */
	private final static Character CONTROL_CHARACTER_SUBSTITUTE = ' ';

	////////////////////
	// constructors
	
	/**
	 * Create an exception object!
	 * 
	 * @param fileName The name of the offending file.
	 * @param lineNumber The line number at which the character was encountered.
	 * @param offender The offending character.
	 * 
	 * @since 2011-05-24
	 */
	public IllegalCharacterException(String fileName, int lineNumber, char offender) {
		// replace control characters by a substitute
		// also, convert from char to int, as FORMAT() doesn't accept chars in combination with the %x format specifier
		super(fileName, lineNumber, String.format("Illegal character '%c' (0x%04X)", Character.isISOControl(offender) ? CONTROL_CHARACTER_SUBSTITUTE : offender, (int)offender));
	}
	
}
