/*
 Copyright (c) 2007--2011 Technische Universitt Darmstadt.
 
 Permission is hereby granted to use, copy, modify, and distribute this
 software, provided that this copyright notice remains intact.
 */

package gsi.rf.data.csv.parser;

/**
 * This class represents a token in a CSV files.
 * 
 * @author Christopher Spies, TU Darmstadt
 * @since 2011-09-09
 * @version 0.1
 */
public class Token {

	////////////////////
	// private constants
	
	////////////////////
	// private members
	
	/**
	 * The character string making up the token.
	 */
	private String data;
	
	/**
	 * The name of the file from which the token was read.
	 */
	private String fileName;
	
	/**
	 * The kind of token represented by this object.
	 */
	private TokenKind kind;
	
	/**
	 * The line number at which this token starts.
	 */
	private int lineNumber;
	
	////////////////////
	// constructors
	
	/**
	 * Create a new token!
	 * 
	 * @param fileName The name of the file from which the token was read.
	 * @param lineNumber The line number from which the token was read.
	 * @param kind The token kind.
	 * @param data The character sequence making up the token.
	 */
	public Token(String fileName, int lineNumber, TokenKind kind, String data) {
		this.fileName = fileName;
		this.lineNumber = lineNumber;
		this.kind = kind;
		this.data = data;
	}
	
	////////////////////
	// public methods
	
	/**
	 * Get the name of the file from which this token was read!
	 * 
	 * @return The file name.
	 * 
	 * @since 2011-09-12
	 */
	public String getFileName() { return this.fileName; }
	
	/**
	 * Determine which kind of token this is!
	 * 
	 * @return The token kind.
	 * 
	 * @since 2011-09-09
	 */
	public TokenKind getKind() { return this.kind; }
	
	/**
	 * Get the line number at which this token is located!
	 * 
	 * @return The line number.
	 * 
	 * @since 2011-09-09
	 */
	public int getLineNumber() { return this.lineNumber; }
	
	/**
	 * Get the character string making up the token!
	 * 
	 * @return The character string.
	 * 
	 * @since 2011-09-09
	 */
	public String getText() { return this.data; }
	
	////////////////////
	// private methods
	
	////////////////////
	// local sub-classes

}
