/*
 Copyright (c) 2007--2012 Technische Universitt Darmstadt.
 
 Permission is hereby granted to use, copy, modify, and distribute this
 software, provided that this copyright notice remains intact.
 */

package gsi.rf.data.csv.parser;

/**
 * This enumeration distinguishes different kinds of tokens in a CSV file.
 * 
 * @author Christopher Spies, TU Darmstadt
 * @since 2011-09-09
 * @version 0.1
 */
public enum TokenKind {

	////////////////////////////
	// enumeration members
	
	/**
	 * A sequence of white-space characters.
	 * 
	 * @since 2011-09-09
	 */
	WHITESPACE("white space"),
	
	/**
	 * A single carriage return ('\r') character.
	 * 
	 * @since 2011-09-09
	 */
	CARRIAGE_RETURN("carriage return"),
	
	/**
	 * A single line feed ('\n') character.
	 * 
	 * @since 2011-09-09
	 */
	LINE_FEED("line feed"),
	
	/**
	 * A line break, a combination of carriage return and line feed characters.
	 * 
	 * @see #CARRIAGE_RETURN
	 * @see #LINE_FEED
	 * 
	 * @since 2011-09-09
	 */
	LINE_BREAK("line break"),
	
	/**
	 * A quotation mark.
	 * 
	 * @since 2011-09-09
	 */
	QUOTATION_MARK("quotation mark"),
	
	/**
	 * Dual quotation marks, the escape sequence for quotation marks inside escaped fields.
	 * 
	 * @see #QUOTATION_MARK
	 * 
	 * @since 2011-09-09
	 */
	ESCAPE_SEQUENCE("dual quotation marks"),
	
	/**
	 * A field separator (comma, semicolon, ...)
	 * 
	 * @since 2011-09-09
	 */
	FIELD_SEPARATOR("field separator"),
	
	/**
	 * Any other accepted non-whitespace character.
	 * 
	 * @since 2011-09-09
	 */
	TEXT_DATA("text data");
	
	////////////////////////////
	// properties of each member
	
	/**
	 * A human-readable description for each token kind.
	 */
	private final String description;
	
	/**
	 * Create a new quantity type!
	 * 
	 * @param code The new quantity's type code.
	 * @param name The new quantity's name.
	 * @param unit The new quantity's default unit.
	 */
	private TokenKind(String description) {
		this.description = description;
	}
	
	/**
	 * Get a human-readable description for this token kind!
	 * 
	 * @return The description.
	 * 
	 * @since 2011-09-12
	 */
	public String getDescription() { return this.description; }
}