/*
 Copyright (c) 2010 Technische Universitt Darmstadt.
 
 Permission is hereby granted to use, copy, modify, and distribute this
 software, provided that this copyright notice remains intact.
 */

package ptolemy.batch;

import java.util.Locale;

import tcl.lang.Interp;
import tcl.lang.TclException;

/**
This application opens a Tcl script specified on the command line.
It goes on to execute this Tcl script by means of Jacl,
a Tcl interpreter written in Java that can be extended by application-specific commands.

@author Christopher Spies
*/
public final class PtolemyBatchApplication {
    
    public static void main(String[] args) {
        Interp tclInterpreter;
        Locale defaultLocale;
        
        if (args.length != 1) {
            System.err.println("Usage:");
            System.err.println("PtolemyBatchApplication <script>");
            System.err.println("  <script> --- the file name of a Tcl script.");
            System.exit(1); // not OK!
        }
        // set locale to US in order to avoid I18n issues within Ptolemy:
        defaultLocale = Locale.getDefault();
        Locale.setDefault(Locale.US);
        
        // set up Tcl interpreter:
        tclInterpreter = new Interp();
        // set up custom commands:
        tclInterpreter.createCommand("ptloadmodel"   , new PtLoadModelCommand());
        tclInterpreter.createCommand("ptgetparameter", new PtGetParameterCommand());
        tclInterpreter.createCommand("ptsetparameter", new PtSetParameterCommand());
        tclInterpreter.createCommand("ptgenerate"    , new PtGenerateCommand());
        tclInterpreter.createCommand("ptrunmodel"    , new PtRunModelCommand());
        // run script:
        try {
            tclInterpreter.evalFile(args[0]);
        } catch (TclException tclex) {
            System.err.print("Error executing script: "); System.err.println(tclex.getMessage());
            tclex.printStackTrace();
            System.err.print("At "); System.err.print(args[0]); System.err.print(", line "); System.err.println(tclInterpreter.getErrorLine());
            System.exit(1); // not OK!
        }
        Locale.setDefault(defaultLocale); // undo changes
        System.exit(0); // OK!
    }

}
